#include "httpconnection.h"
#include "filelog.h"
#include <iostream>
#include <cstdio>
using namespace std;

HttpConnection::~HttpConnection()
{
	delete this->request;
	delete this->response;
	if(this->clientSocket)
		delete this->clientSocket;
	if(this->remoteSocket)
		delete this->remoteSocket;
}

HttpConnection::HttpConnection(HttpSocket* __sock)
{
	this->clientSocket = __sock;
	this->remoteSocket = 0;
	this->request = new HttpRequest();
	this->response = new HttpResponse();
	this->closed = false;
	this->minBlockSize = 64*1024;
	this->maxBlockSize = 4*1024*1024;
}

void HttpConnection::process()
{
	string host = "";
	int port = 0;
	for(int c=1; !this->closed; c++){
		if(!this->request->parseRequest(this->clientSocket))
			break;
		
		cout << this->request->getCommand() << " " << this->request->getPath() << 
			" " << this->request->getHost() <<  endl;
			
		/* Check if need to re-establish connection */
		if(this->remoteSocket && host != this->request->getHost() &&
			port != this->request->getPort()){
			port = this->request->getPort();
			delete this->remoteSocket;
			this->remoteSocket = 0;
		}
		
		/* Connect to remote server */
		if(this->remoteSocket == 0){
			this->remoteSocket = new HttpSocket();
			if(!this->remoteSocket->connect(this->request->getHost(), 
				this->request->getPort()))
					break;
		}
		
		/* Do CONNECT */
		if(this->request->getCommand() == "CONNECT"){
			this->clientSocket->send("HTTP/1.1 200 Established\r\n");
			this->doCONNECT();
			break;
		}
		
		/* Retry */
		if(!this->request->sendRequest(this->remoteSocket)){
			delete this->remoteSocket;
			this->remoteSocket = new HttpSocket();
			if(!this->remoteSocket->connect(this->request->getHost(), 
				this->request->getPort()))
					break;
			if(!this->request->sendRequest(this->remoteSocket))
				break;
		}
		
		/* Get Response */
		if(!this->response->parseResponse(this->remoteSocket))
			break;
		
		/* Send Response */
		if(!this->response->sendResponse(this->clientSocket))
			break;
		
		/* Response Data */
		if(this->response->getContentLength() > 0){
			if(this->response->isChunked()){
				this->transferTrunkedData();
			}else{
				/* Test */
				if(this->isSpeedable()){
					this->speeded = true;
					this->transferDataSpeeded();
				}else{
					this->speeded = false;
					this->transferData();
				}
			}
		}
		
//		cout << "handled connections: " << c << endl;
		if(this->response->wouldClose() || this->request->wouldClose())
			break;
	}
	
	if(this->clientSocket)
		this->clientSocket->shutdown();
	if(this->remoteSocket)
		this->remoteSocket->shutdown();
	this->closed = true;
}

void HttpConnection::doCONNECT()
{
	const int bufferSize = 8192;
	char* buffer = new char[bufferSize];
	if(buffer == NULL)
		return;
	fd_set fdreads;
	int sock1 = this->clientSocket->getSocketHandle();
	int sock2 = this->remoteSocket->getSocketHandle();
	int ret, size;
	for(;!this->closed;){
		FD_ZERO(&fdreads);
		FD_SET(sock1, &fdreads);
		FD_SET(sock2, &fdreads);
		ret = select((sock1>sock2 ? sock1:sock2)+1, &fdreads, 0, 0, 0);
		if(ret <= 0)
			break;
		else{
			if(FD_ISSET(sock1, &fdreads)){
				size = recv(sock1, buffer, bufferSize, 0);
				if(size <= 0)
					break;
				send(sock2, buffer, size, 0);
			}
			if(FD_ISSET(sock2, &fdreads)){
				size = recv(sock2, buffer, bufferSize, 0);
				if(size <= 0)
					break;
				send(sock1, buffer, size, 0);
			}
		}
	}
	delete buffer;
}

void HttpConnection::close()
{
	if(this->clientSocket)
		this->clientSocket->shutdown();
	if(this->remoteSocket)
		this->remoteSocket->shutdown();
}

bool HttpConnection::isClosed() const
{
	return this->closed;
}


bool HttpConnection::transferData()
{
	unsigned int length = this->response->getContentLength();
	const int bufferSize = 8192;
	char* buffer = new char[bufferSize];
	if(buffer == NULL)
		return false;
	while(length > 0){
		int sendSize = length < bufferSize ? length : bufferSize;
		if((sendSize = this->remoteSocket->receive(buffer, sendSize)) <= 0)
			break;
		if(this->clientSocket->send(buffer, sendSize) != sendSize)
			break;
		length -= sendSize;
	}
	delete buffer;
}

bool HttpConnection::transferTrunkedData()
{
	const int bufferSize = 8192;
	char* buffer = new char[bufferSize];
	if(buffer == NULL)
		return false;
	string line;
	bool end = false;
	for(;!end;){
		/* read trunk size */
		if(!this->remoteSocket->receive(line))
			break;
		
		unsigned int length = 0;
		sscanf(line.c_str(), "%x", &length);
		if(length == 0)
			end = true;

		while(length > 0){
			int sendSize = length < bufferSize ? length : bufferSize;
			if((sendSize = this->remoteSocket->receive(buffer, sendSize)) <= 0)
				break;
			if(this->clientSocket->send(buffer, sendSize) != sendSize)
				break;
			length -= sendSize;
		}
		
		/* read endline */
		if(!this->remoteSocket->receive(line))
			break;
		if(!line.empty())
			break;
	}
	
	delete buffer;
	return true;
}

bool HttpConnection::transferDataSpeeded()
{
	cout << "transferDataSpeeded()" << endl;
	/* initialize state */
	this->contentLength = this->response->getContentLength();
	this->currentPosition = 0;
	this->startTime = GetTickCount();
	this->remoteSocket->shutdown();
	
	/* first buffer */
	ContentBuffer* cb;
	this->blockSize = minBlockSize;
	cb = new ContentBuffer(this->blockSize);
	cb->dataBegin = 0;
	cb->dataEnd = cb->dataBegin + cb->bufferSize;
	this->buffers.push_back(cb);
	this->currentPosition += cb->bufferSize;
	
	/* first shrimp */
	HttpShrimp* shrimp = new HttpShrimp(this, cb);
	this->shrimps.push_back(shrimp);
	shrimp->startThread();
	
	for(;this->currentPosition < this->contentLength || cb;){
		while(cb->readPos < cb->writePos){
			int sendSize = cb->writePos - cb->readPos;
			if(sendSize > 8192)
				sendSize = 8192;
			if(this->clientSocket->send(&cb->buffer[cb->readPos], sendSize) 
				!= sendSize)
				goto BED;
			cb->readPos += sendSize;
		}
		
		if(cb->readPos == cb->bufferSize && 
				cb->worker == 0/* no shrimp on it*/){
			cout << "Buffering %" << 100 * cb->dataEnd / this->contentLength << "[" << 
				cb->dataEnd << "/" << this->contentLength << "]" << endl;
			
			this->lock();
			delete cb;
			this->buffers.erase(this->buffers.begin());
			if(this->buffers.size() > 0)
				cb = *this->buffers.begin();
			else
				cb = 0;
			this->unlock();
		}
		
		Sleep(10);
	}
	
BED:
	unsigned int seconds = ((GetTickCount() - this->startTime)/1000);
	if(seconds)
		cout << "Transfer completed, avarage speed: " << 
			(this->contentLength/1000) / seconds << "kB/s" << endl;
		
	this->lock();
	/* cleanup all buffers & shrimps */
	for(vector<HttpShrimp*>::iterator it = this->shrimps.begin(); 
		it != this->shrimps.end(); it++){
//		cout << "kill " << (*it)->getShrimpId() << endl;
		(*it)->kill();
	}
	this->unlock();
	
	for(vector<HttpShrimp*>::iterator it = this->shrimps.begin(); 
		it != this->shrimps.end(); it++){
		(*it)->joinThread();
		delete *it;
	}
	
	for(vector<ContentBuffer*>::iterator it = this->buffers.begin();
		it !=this->buffers.end(); it++)
		delete *it;
//	cout << "transferDataSpeeded() end" << endl;
	return true;
}

HttpRequest* HttpConnection::getRequest()
{
	return this->request;
}

HttpResponse* HttpConnection::getResponse()
{
	return this->response;
}

void HttpConnection::updateBlockSize(int __size)
{
	this->blockSize = __size;
	if(this->blockSize > this->contentLength - this->currentPosition)
		this->blockSize = this->contentLength - this->currentPosition;
}

bool HttpConnection::isSpeedable()
{
	const static char accept_types[][16] = {
		"video",
		"media",
		"image",
		"application",
		"audio"
	};
	if(this->request->getCommand() == "GET" && 
		this->response->getStatus() == "200" &&
		this->response->getContentLength() > this->minBlockSize){
		string type = this->response->getMessage().getHeader("content-type");
		for(int i=0; i<sizeof(accept_types) / 16; i++)
			if(type.find(accept_types[i])!=string::npos)
				return true;
	}
	return false;
}

