#include <windows.h>
#include "httpshrimp.h"
#include "httpconnection.h"
#include <iostream>
#include <cstdio>

static int shrimpIdGenerator = 0;

HttpShrimp::~HttpShrimp()
{
	if(this->remoteSocket)
		delete this->remoteSocket;
}

HttpShrimp::HttpShrimp(HttpConnection* __conn, ContentBuffer* __cb)
{
	this->connection = __conn;
	this->remoteSocket = 0;
	this->dead = false;
	this->contentBuffer = __cb;
	if(__cb)
		__cb->worker = this;
	this->request = this->connection->getRequest();
	this->response = new HttpResponse();
	this->shrimpId = shrimpIdGenerator++;
}

void HttpShrimp::process()
{
	ContentBuffer* cb = this->contentBuffer;
	if(cb){
		if(!this->establishConnection(cb))
			goto BED;
		cout << "shrimp begin at " << cb->dataBegin << endl;
	}
	for(;!this->dead; ){
		if(cb == 0){
			this->connection->lock();
			/* Find a block */
			for(vector<ContentBuffer*>::iterator it = this->connection->buffers.begin();
				it != this->connection->buffers.end(); it++){
				if((*it)->worker == 0 && (*it)->writePos == 0){
					cb = *it;
					cb->worker = this;
					break;
				}
			}
			if(cb == 0){
				/* Create a new block */
				unsigned int size = this->connection->contentLength - 
					this->connection->currentPosition;
				if(size == 0){
					this->connection->unlock();
					break;	
				}
				
				size = size > this->connection->blockSize ? 
					this->connection->blockSize : size;
					
				cb = new ContentBuffer(size);
				cb->dataBegin = this->connection->currentPosition;
				cb->dataEnd = cb->dataBegin + size;
				cb->worker = this;
				this->connection->buffers.push_back(cb);
				
				this->connection->currentPosition += size;
			}
			/* reconnect to the server */
			if(!this->establishConnection(cb)){
				cb->worker = 0;
				cb->writePos = 0;
				cb = 0;
				Sleep(100);
			}
			this->connection->unlock();
		}else{
			int ret;
			if((ret = this->remoteSocket->receive(&cb->buffer[cb->writePos], 
				cb->bufferSize - cb->writePos)) <= 0){
				cb->writePos = 0;
				cb->worker = 0;
				continue;
			}
//			cout << "shrimp recv " << ret << " bytes at " << cb->dataBegin +
//				cb->writePos << endl;
			cb->writePos += ret;
			this->transferedBytes += ret;
			
			/* Finish one block */
			if(cb->writePos == cb->bufferSize){
				if(cb->dataBegin == 0){
					/* Is the first block */
					unsigned int speed = this->calculateSpeed();
					unsigned int blockSize = speed * 8;
					cout << "updateBlockSize " << blockSize << endl;
					this->connection->updateBlockSize(blockSize);
					this->connection->lock();
					for(int i=0; i<7 && i*blockSize < this->connection->contentLength; i++)
						this->fork();
					this->connection->unlock();
				}
				
				cb->worker = 0;
				ContentBuffer* oldCb = cb;
				cb = 0;
				
				this->connection->lock();
				/* check the next block */
				for(vector<ContentBuffer*>::iterator it = this->connection->buffers.begin();
					it != this->connection->buffers.end(); it++){
					if((*it) == oldCb	){
						it++;
						if(it == this->connection->buffers.end())
							break;
						if((*it)->writePos == 0){
							HttpShrimp* shrimp = (*it)->worker;
							if(shrimp){
								shrimp->lock();
								if(shrimp->remoteSocket)
									shrimp->remoteSocket->shutdown();
								shrimp->unlock();
							}
							cb = *it;
							(*it)->worker = this;
							cout << "Shrimp " << this->shrimpId << 
								" occupied at " << cb->dataBegin << endl;
							(*it)->writePos = 0;
							break;
						}
					}
				}
				this->connection->unlock();
			}
		}
	}
	cout << "shrimp " << this->shrimpId << 
		" died. Its speed is " << this->calculateSpeed()/1000 << "kB/s" << endl;
BED:
	this->dead = true;
}

void HttpShrimp::kill()
{
	this->dead = true;
	if(this->remoteSocket)
		this->remoteSocket->shutdown();
}

bool HttpShrimp::establishConnection(ContentBuffer* __cb)
{
	if(this->remoteSocket != 0){
		this->remoteSocket->shutdown();
		this->lock();
		delete this->remoteSocket;
		this->remoteSocket = 0;
		this->unlock();
	}
	this->remoteSocket = new HttpSocket();
	if(!this->remoteSocket->connect(request->getHost(), request->getPort())){
		cout << "failed to connect to " << request->getHost() << ":" <<
			request->getPort()<<endl;
		return false;
	}
	if(__cb->dataBegin > 0){
		char rangestr[32];
		sprintf(rangestr, "bytes=%u-", __cb->dataBegin);
		request->getMessage().setHeader("Range", rangestr);
	}
	if(!request->sendRequest(this->remoteSocket))
		return false;
	
	if(!response->parseResponse(this->remoteSocket))
		return false;
	
	if(response->getStatus() != "200" && response->getStatus() != "206")
		return false;
	
	if(response->getContentLength() < __cb->bufferSize)
		return false;
	
	this->startTime = GetTickCount();
	this->transferedBytes = 0;
	return true;
}

unsigned int HttpShrimp::calculateSpeed()
{
	unsigned int endTime = GetTickCount();
	unsigned int timeElapsed = endTime - startTime;
	
	if(timeElapsed == 0)
		return 0;
	
	/* bytes / ms */
	double speed = (double)this->transferedBytes / timeElapsed;
	
	return speed * 1000;
}

void HttpShrimp::fork()
{
	HttpShrimp* shrimp = new HttpShrimp(this->connection, 0);
	this->connection->shrimps.push_back(shrimp);
	shrimp->startThread();
}

int HttpShrimp::getShrimpId()
{
	return this->shrimpId;
}

